@@ -1,5 +1,5 @@
-#ifndef _CASSETTE_H_
-#define _CASSETTE_H_
+#ifndef CASSETTE_H_
+#define CASSETTE_H_
 
 #include <stdio.h>		/* for FILE and FILENAME_MAX */
 
@@ -7,21 +7,16 @@
 
 #define CASSETTE_DESCRIPTION_MAX 256
 
-void CASSETTE_Initialise(int *argc, char *argv[]);
+int CASSETTE_Initialise(int *argc, char *argv[]);
 
 int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *last_block, int *isCAS);
 int CASSETTE_CreateFile(const char *filename, FILE **fp, int *isCAS);
 int CASSETTE_Insert(const char *filename);
 void CASSETTE_Remove(void);
-extern char cassette_filename[FILENAME_MAX];
-extern char cassette_description[CASSETTE_DESCRIPTION_MAX];
 
-extern int cassette_current_block;
-extern int cassette_max_block;
-
-extern int hold_start;
-extern int hold_start_on_reboot; /* preserve hold_start after reboot */
-extern int press_space;
+extern int CASSETTE_hold_start;
+extern int CASSETTE_hold_start_on_reboot; /* preserve hold_start after reboot */
+extern int CASSETTE_press_space;
 
 int CASSETTE_AddGap(int gaptime);
 void CASSETTE_LeaderLoad(void);
@@ -31,8 +26,10 @@ int CASSETTE_Write(int length);
 int CASSETTE_IOLineStatus(void);
 int CASSETTE_GetByte(void);
 int CASSETTE_GetInputIRQDelay(void);
+int CASSETTE_IsSaveFile(void);
+void CASSETTE_PutByte(int byte);
 void CASSETTE_TapeMotor(int onoff);
 void CASSETTE_AddScanLine(void);
-extern UBYTE cassette_buffer[4096];
+extern UBYTE CASSETTE_buffer[4096];
 
-#endif /* _CASSETTE_H_ */
+#endif /* CASSETTE_H_ */
