@@ -15,4 +15,8 @@ int MZPOKEYSND_Init(ULONG freq17,
 #endif
                        );
 
+#ifdef SYNCHRONIZED_SOUND
+#endif /* SYNCHRONIZED_SOUND */
+int MZPOKEYSND_UpdateProcessBuffer(void);
+extern UBYTE *MZPOKEYSND_process_buffer;
 #endif /* MZPOKEYSND_H_ */
