@@ -1,6 +1,8 @@
 #ifndef SCREEN_H_
 #define SCREEN_H_
 
+#include <stdio.h>
+
 #include "atari.h"  /* UBYTE */
 
 #ifdef DIRTYRECT
@@ -25,6 +27,13 @@ extern ULONG *Screen_atari1;
 extern ULONG *Screen_atari2;
 #endif
 
+/* The area that can been seen is Screen_visible_x1 <= x < Screen_visible_x2,
+   Screen_visible_y1 <= y < Screen_visible_y2.
+   Full Atari screen is 336x240. Screen_WIDTH is 384 only because
+   the code in antic.c sometimes draws more than 336 bytes in a line.
+   Currently Screen_visible variables are used only to place
+   disk led and snailmeter in the corners of the screen.
+*/
 extern int Screen_visible_x1;
 extern int Screen_visible_y1;
 extern int Screen_visible_x2;
@@ -35,9 +44,11 @@ extern int Screen_show_disk_led;
 extern int Screen_show_sector_counter;
 
 int Screen_Initialise(int *argc, char *argv[]);
+int Screen_ReadConfig(char *string, char *ptr);
+void Screen_WriteConfig(FILE *fp);
 void Screen_DrawAtariSpeed(double);
 void Screen_DrawDiskLED(void);
-void Screen_FindScreenshotFilename(char *buffer);
+void Screen_FindScreenshotFilename(char *buffer, unsigned bufsize);
 int Screen_SaveScreenshot(const char *filename, int interlaced);
 void Screen_SaveNextScreenshot(int interlaced);
 void Screen_EntireDirty(void);
